/*=========================================================================*
  G_UnShadedPolygon
 *=========================================================================*/
void G_UnshadedPolygon(Triangle *Poly, char *VPage, int x, int y)
{
  if (PreviousState != (UNSHADED | SMOOTH))
    {
      /* set color combiner and clear texture and alpha combiners */
      PreviousState = (UNSHADED | SMOOTH);
      grColorCombine(GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE,
                     GR_COMBINE_LOCAL_CONSTANT, GR_COMBINE_OTHER_NONE, FXFALSE);

      grAlphaCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_ONE,
                     GR_COMBINE_LOCAL_NONE, GR_COMBINE_OTHER_CONSTANT, FXFALSE);

      grAlphaBlendFunction(GR_BLEND_ONE, GR_BLEND_ZERO,
                           GR_BLEND_ZERO, GR_BLEND_ZERO);
    }

  grConstantColorValue((GlobalPalette[Poly->Color].r << 2) << 16 |
                       (GlobalPalette[Poly->Color].g << 2) << 8  |
                       (GlobalPalette[Poly->Color].b << 2) << 0);

  STD_TRI_SETUP;
  grDrawTriangle(&A, &B, &C);
}

/*=========================================================================*
  G_AmbientPolygon
 *=========================================================================*/
void G_AmbientPolygon(Triangle *Poly, char *VPage, int x, int y)
{
  int   r,g,b;
  float GlossVal;

  grConstantColorValue( (GlobalPalette[Poly->Color].r << 16) |
                        (GlobalPalette[Poly->Color].g << 8) |
                        (GlobalPalette[Poly->Color].b << 0));

  if (PreviousState != (AMBIENT | SMOOTH))
    {
      PreviousState = (AMBIENT | SMOOTH);
      grColorCombine(GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE,
                     GR_COMBINE_LOCAL_CONSTANT, GR_COMBINE_OTHER_NONE, FXFALSE);

      grAlphaCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_ONE,
                     GR_COMBINE_LOCAL_NONE, GR_COMBINE_OTHER_CONSTANT, FXFALSE);

      grAlphaBlendFunction(GR_BLEND_ONE, GR_BLEND_ZERO,
                           GR_BLEND_ZERO, GR_BLEND_ZERO);
    }

  GlossVal = pSPECULAR*(GlossTable[FLT2INT(Poly->Shade * 1023)]);

  r = (GlobalPalette[Poly->Color].r << 2);
  r = FLT2INT(pAMBIENT*((float)r*2.0/5.0) + pDIFFUSE*((float)r*Poly->Shade) + GlossVal);

  g = (GlobalPalette[Poly->Color].g << 2);
  g = FLT2INT(pAMBIENT*((float)g*2.0/5.0) + pDIFFUSE*((float)g*Poly->Shade) + GlossVal);

  b = (GlobalPalette[Poly->Color].b << 2);
  b = FLT2INT(pAMBIENT*((float)b*2.0/5.0) + pDIFFUSE*((float)b*Poly->Shade) + GlossVal);

  grConstantColorValue((r << 16) | (g << 8) | (b));

  STD_TRI_SETUP;
  grDrawTriangle(&A, &B, &C);
}

/*=========================================================================*
  G_LambertPolygon
 *=========================================================================*/
void G_LambertPolygon(Triangle *Poly, char *VPage, int x, int y)
{
  float GlossVal;
  int r, g, b;

  if (PreviousState != (LAMBERT | SMOOTH))
    {
      PreviousState = (LAMBERT | SMOOTH);
      grColorCombine(GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE,
                     GR_COMBINE_LOCAL_CONSTANT, GR_COMBINE_OTHER_NONE, FXFALSE);

      grAlphaCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_ONE,
                     GR_COMBINE_LOCAL_NONE, GR_COMBINE_OTHER_CONSTANT, FXFALSE);

      grAlphaBlendFunction(GR_BLEND_ONE, GR_BLEND_ZERO,
                           GR_BLEND_ZERO, GR_BLEND_ZERO);
    }

  GlossVal = pSPECULAR*(GlossTable[FLT2INT(Poly->Shade * 1023)]);

  r = (GlobalPalette[Poly->Color].r << 2);
  r = FLT2INT(pAMBIENT*((float)r*2.0/5.0) + pDIFFUSE*((float)r*Poly->Shade) + GlossVal);

  g = (GlobalPalette[Poly->Color].g << 2);
  g = FLT2INT(pAMBIENT*((float)g*2.0/5.0) + pDIFFUSE*((float)g*Poly->Shade) + GlossVal);

  b = (GlobalPalette[Poly->Color].b << 2);
  b = FLT2INT(pAMBIENT*((float)b*2.0/5.0) + pDIFFUSE*((float)b*Poly->Shade) + GlossVal);

  grConstantColorValue((r << 16) | (g << 8) | (b));

  STD_TRI_SETUP;
  grDrawTriangle(&A, &B, &C);
}

/*=========================================================================*
  G_LambertPolygon
 *=========================================================================*/
void G_GouraudPolygon(Triangle *Poly, char *VPage, int x, int y)
{
  float GlossVal;
  float r, g, b;

  if (PreviousState != (GOURAUD | SMOOTH))
    {
      PreviousState = (GOURAUD | SMOOTH);
      grColorCombine(GR_COMBINE_FUNCTION_LOCAL, GR_COMBINE_FACTOR_NONE,
                     GR_COMBINE_LOCAL_ITERATED, GR_COMBINE_OTHER_NONE, FXFALSE);

      grAlphaCombine(GR_COMBINE_FUNCTION_SCALE_OTHER, GR_COMBINE_FACTOR_ONE,
                     GR_COMBINE_LOCAL_NONE, GR_COMBINE_OTHER_CONSTANT, FXFALSE);

      grAlphaBlendFunction(GR_BLEND_ONE, GR_BLEND_ZERO,
                           GR_BLEND_ZERO, GR_BLEND_ZERO);
    }



  r = (GlobalPalette[Poly->Color].r << 2);
  g = (GlobalPalette[Poly->Color].g << 2);
  b = (GlobalPalette[Poly->Color].b << 2);

  GlossVal = pSPECULAR*(GlossTable[FLT2INT(Poly->A->Shade * 1023)]);
  A.r = (pAMBIENT*(r*2.0/5.0) + pDIFFUSE*(r*Poly->A->Shade) + GlossVal);
  A.g = (pAMBIENT*(g*2.0/5.0) + pDIFFUSE*(g*Poly->A->Shade) + GlossVal);
  A.b = (pAMBIENT*(b*2.0/5.0) + pDIFFUSE*(b*Poly->A->Shade) + GlossVal);

  GlossVal = pSPECULAR*(GlossTable[FLT2INT(Poly->B->Shade * 1023)]);
  B.r = (pAMBIENT*(r*2.0/5.0) + pDIFFUSE*(r*Poly->B->Shade) + GlossVal);
  B.g = (pAMBIENT*(g*2.0/5.0) + pDIFFUSE*(g*Poly->B->Shade) + GlossVal);
  B.b = (pAMBIENT*(b*2.0/5.0) + pDIFFUSE*(b*Poly->B->Shade) + GlossVal);

  GlossVal = pSPECULAR*(GlossTable[FLT2INT(Poly->C->Shade * 1023)]);
  C.r = (pAMBIENT*(r*2.0/5.0) + pDIFFUSE*(r*Poly->C->Shade) + GlossVal);
  C.g = (pAMBIENT*(g*2.0/5.0) + pDIFFUSE*(g*Poly->C->Shade) + GlossVal);
  C.b = (pAMBIENT*(b*2.0/5.0) + pDIFFUSE*(b*Poly->C->Shade) + GlossVal);

  STD_TRI_SETUP;
  grDrawTriangle(&A, &B, &C);
}

void G_PhongPolygon(Triangle *Poly, char *VPage, int x, int y)
{
}
