#ifndef __INERTIA_H__
#define __INERTIA_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "types.h"

#ifndef PI
#define PI                   3.141592f
#endif

extern "C" float _tmp_flt;

#ifdef __WATCOM_C__
int FLT2INT(float a);
#pragma aux FLT2INT =        \
"fistp dword ptr [_tmp_flt]" \
"Mov   eax,[_tmp_flt]"       \
parm  [8087]                 \
value [eax];
#else
#define FLT2INT(a)           ((int)(a))
#endif

#define FLT2FXD(a)           (FLT2INT((a) * 65536.0f))

#define CLIP(a,b,c)          ((a)<(b)?(b):(((a)>(c)?(c):(a))))
#define SWAP(a,b,c)          {c = a; a = b; b = c;}
#define SQR(a)               ((a)*(a))
#define ABS(a)               ((a)<0?-(a):(a))
#define DEG2RAD(a)           ((a) * PI / 180.0)
#define DOTPRODUCT(a,b)      ((a).x * (b).x + (a).y * (b).y + (a).z * (b).z)


// Some globally accessable variables from the Assembler code
extern "C" char             *IntensityTable;
extern "C" char             *TransparencyTable;


extern ShaderFunction   Shaders[256];
extern TriangleFiller   TriFillers[256];
extern float            AspectRatio;
extern tLightList       Lights;
extern Palette          GlobalPalette;
extern tRenderList      RenderList;
extern TextureRec       TextureList[MAXTEXTURES];
extern int              GlideFlag;

void InitInertia();
void ShutDownInertia();
void InitVectorObject(VectorObject *Obj);
int  AllocObject(VectorObject *Obj, int NumVert, int NumPoly);
void FreeVectorObject(VectorObject *Obj);
int  CopyVectorObject(VectorObject *Source, VectorObject *Dest);
void CalcBoundaryVolume(VectorObject *Obj);
int  LoadGVO(VectorObject *Obj, char *Filename, int FOfs);
void SetAbsoluteRotation(VectorObject *Obj, float rx, float ry, float rz);
void SetDeltaRotation(VectorObject *Obj, float dx, float dy, float dz);
void SetAbsoluteLocation(VectorObject *Obj, float tx, float ty, float tz);
void SetDeltaLocation(VectorObject *Obj, float dx, float dy, float dz);
void SetBaseColor(VectorObject *Obj, char C);
void SetPolygonColor(VectorObject *Obj, int Poly, char C);
void SetObjectTexture(VectorObject *Obj, int C);
void SetPolygonTexture(VectorObject *Obj, int Poly, int C);
void SetObjectPhongMap(VectorObject *Obj, int C);
void SetPolygonPhongMap(VectorObject *Obj, int Poly, int C);
void SetRenderMask(VectorObject *Obj, int Mask);
void SetCull(VectorObject *Obj, int Cull);
void SetAspectRatio(float ARatio);
void SetFocalLength(ViewObject *View, float fl);
void SetFieldOfView(ViewObject *View, float FOV);
void InitView(ViewObject *View);
void FreeView(ViewObject *View);
void GetObjectAsCamera(ViewObject *View, VectorObject *Obj);
void SetViewFustrum(ViewObject *View, float MinZ, float MaxZ);
void SetViewPort(ViewObject *View, float X1, float Y1, float X2, float Y2, char Update);
void SetViewLocation(ViewObject *View, float X, float Y, float Z);
void SetFocusPoint(ViewObject *View, float TargetX, float TargetY, float TargetZ, float Roll);
void PushLightSource(float X, float Y, float Z);
void PopLightSource();
void RotateLightSource(int Index, float Xa, float Ya, float Za);
void ByteSort();
void Render(char *VPage, int x, int y);
int  ObjectVisible(ViewObject *View, VectorObject *Obj, float *M, float Dot);
void AddToRenderList(ViewObject *View, VectorObject *Obj);
int  LoadIntensityTable(char *Filename, int FOfs);
void FreeIntensityTable();
int  LoadTransparencyTable(char *Filename, int FOfs);
void FreeTransparencyTable();
int  LoadPalette(char *Filename, int FOfs);
void FreeTexture(int Index);
int  LoadPCXTexture(int Index, char *Filename, int FOfs);
void GetTexturePalette(int Index);
int  MakePhongMap(int Index);

#ifdef __USE_GLIDE__

int  G_InitInertia();
int  G_OpenWindow();
void G_Shutdown();
void G_SetPalette();
void G_FreeTexture(char *Texture);
void G_Phong(float a, float d, float s, float Gloss);
void G_Flip();

#endif

#endif
