;
;                    Inertia Realtime 3D Rendering Engine
;    Copyright (c) 1996, Alex Chalfin, Jeroen Bouwens. All Rights Reserved.
;          DISTRIBUTION OF THIS SOURCE CODE IS STRICTLY PROHIBITED
;

;
; POLYFLSM.ASM
; code: Alex Chalfin
;

; Flat (Lambert) polygon filler for smooth surface model
;
;================================================================
; Polygon specific defines
;================================================================
;
;__INTERP_EXTRA__  EQU 0        ; must be defined for everything but flat
;__INTERP_C__      EQU 0        ; must be defined for goruaud shading
;__INTERP_UV__     EQU 0        ; must be defined for texture mapping
;__INTERP_ENV__    EQU 0        ; must be defined for environment mapping
;

.386p                           ; 486 protected mode
.387
.Model Flat,C                   ; with flat memory model

DATA SEGMENT PARA PUBLIC USE32  ;
                                ;
public _FlatPolygonASM          ;
                                ;
include strucdef.asm            ;  Inertia structure references
include varsasm.inc             ; external variable references
                                ;
DATA ENDS                       ;
                                ;
;================================================================
; Code Segment
;================================================================
                                ;
CODE SEGMENT PARA PUBLIC USE32  ;
                                ;
;===========================================================================
; Macros
;===========================================================================
                                ;
                                ; All registers are available for loop
                                ; macro. Nothing needs to be saved/restored
                                ; it is all handled in AFFASM.INC
                                ;
INNER_LOOP Macro  Left, Right, Height, Side;
Local  @@HeightLooper,@@ExitHeight,@@SkipFill ; Local labels
                                ;
 @@HeightLooper:                ;
  Mov  eax,[Height]             ;
  Cmp  eax,0                    ;
  Jle @@ExitHeight              ;
                                ;
  Dec  [Height]                 ;  decrement the height value by 1
                                ;
  Mov  eax,[Left&X]             ; grab the left and the right values
  Mov  ecx,[Right&X]            ;
                                ;
  Add  eax,65535                ;
  Dec  ecx                      ;
                                ;
  Sar  eax,16                   ; hx1 = (LeftX + 65535) >> 16
  Sar  ecx,16                   ; hx2 = (RightX - 1) >> 16
                                ;
  Sub  ecx,eax                  ;
  Inc  ecx                      ; ecx = hx2 - hx1 + 1
                                ;
  Cmp  ecx,0                    ;
  Jle @@SkipFill                ;
								;
  Mov  edi,[_YAddress]          ;
  Add  edi,eax                  ; edi is virtual screen address
                                ;
  Mov  eax,[_Color]             ; Color for the polygon
                                ;
  Mov  edx,ecx                  ;
  Shr  ecx,2                    ;
  And  edx,3                    ;
                                ;
  Rep  Stosd                    ;
                                ;
  Mov  ecx,edx                  ;
                                ;
  Rep  Stosb                    ;
                                ;
 @@SkipFill:                    ;
                                ;
  Mov  edx,[_YAddress]           ; go to the next scanline
  Add  edx,_LineSize           ;
  Mov  [_YAddress],edx           ; YAddress += LineSize
                                ;
  Mov  eax,[Left&X]             ;
  Mov  ebx,[Right&X]            ;
  Add  eax,[Left&DX]            ;
  Add  ebx,[Right&DX]           ;
  Mov  [Left&X],eax             ; LeftX += LeftDX
  Mov  [Right&X],ebx            ; RightX += RightDX
                                ;
  Jmp @@HeightLooper            ;
                                ;
 @@ExitHeight:                  ;
EndM                            ;
                                ;
;===========================================================================
; CullPolys
;===========================================================================
_FlatPolygonASM Proc Uses edi eax,_Triangle:DWord, _VirtualAddress:DWord, _LineSize:DWord
                                ;
  Pushad						;
  Mov  edi,_Triangle            ;
                                ;
  fld  dword ptr [edi].PShade   ;
  fmul dword ptr [__tmp_flt_63] ;
                                ;
  fistp dword ptr [_Color]      ;
  Mov   eax,[_Color]            ;
  Shl   eax,8                   ;
  Add   eax,[edi].pColor        ;
                                ;
  Mov   edx,[_IntensityTable]   ;
  Mov   ebx,[edx + eax]         ;
                                ;
  And   ebx,0ffh                ;
                                ;
  Xor   eax,eax                 ;
  Mov   bh,bl                   ;
  Mov   eax,ebx                 ;
  Shl   ebx,16                  ;
  Or    ebx,eax                 ;
                                ;
  Mov   [_Color],ebx            ;
                                ;
include affasm.inc              ; include affine interpolation
                                ;
  Popad                         ;
								;
  Ret                           ;
                                ;
_FlatPolygonASM EndP            ;


CODE ENDS
END


