;
;                    Inertia Realtime 3D Rendering Engine
;    Copyright (c) 1996, Alex Chalfin, Jeroen Bouwens. All Rights Reserved.
;          DISTRIBUTION OF THIS SOURCE CODE IS STRICTLY PROHIBITED
;

;
; POLYGOSM.ASM
; code: Alex Chalfin
;

; Gouraud polygon filler for smooth surface model
;
;================================================================
; Polygon specific defines
;================================================================
;
__INTERP_EXTRA__  EQU 0        ; must be defined for everything but flat
__INTERP_C__      EQU 0        ; must be defined for goruaud shading
;__INTERP_UV__     EQU 0        ; must be defined for texture mapping
;__INTERP_ENV__    EQU 0        ; must be defined for environment mapping
;

.386p                           ; 486 protected mode
.387                            ;
.Model Flat,C                   ; with flat memory model

DATA SEGMENT PARA PUBLIC USE32  ;
                                ;
public _GouraudPolygonASM        ;
                                ;
include strucdef.asm            ;  Inertia structure references
include varsasm.inc             ; external variable references
                                ;
DATA ENDS                       ;
                                ;
;================================================================
; Code Segment
;================================================================
                                ;
CODE SEGMENT PARA PUBLIC USE32  ;
                                ;
;===========================================================================
; Macros
;===========================================================================
                                ;
                                ; All registers are available for loop
                                ; macro. Nothing needs to be saved/restored
                                ; it is all handled in AFFASM.INC
                                ;
INNER_LOOP Macro  Left, Right, Height, Side;
Local  @@HeightLooper,@@ExitHeight,@@SkipFill,@@FillLoop ; Local labels
                                ;
 @@HeightLooper:                ;
  Mov  eax,[Height]             ;
  Cmp  eax,0                    ;
  Jle @@ExitHeight              ;
                                ;
  Dec  [Height]                 ;  decrement the height value by 1
                                ;
  Mov  ebx,[Left&X]             ;
  Add  ebx,65535                ;
  And  ebx,0ffff0000h           ;
                                ;
  Mov  edx,ebx                  ;
  Sub  edx,[Left&X]             ;
                                ;
  Mov  eax,[_HorizontalDC]       ;
  IMul edx                      ;
  Shrd eax,edx,16               ; fix for sub-pixel adjustment
                                ;
  Add  eax,[Side&C]             ; eax is the starting horizontal color
                                ;
  Sar  ebx,16                   ;
                                ;
  Mov  ecx,[Right&X]            ;
  Dec  ecx                      ;
  Sar  ecx,16                   ;
                                ;
  Sub  ecx,ebx                  ;
  Inc  ecx                      ;
  Cmp  ecx,0                    ;
  Jle @@SkipFill                ;
                                ;
  Mov  edi,[_YAddress]           ;
  Add  edi,ebx                  ;
                                ;
  Mov  esi,[_IntensityTable]     ;
  Add  esi,[_Color]              ;
                                ; On entering loop:
                                ; edi = screen position
                                ; eax = 16.16 gouraud intensity
                                ; ecx = count
                                ; esi = shade table base
 @@FillLoop:                    ;
  Mov  ebx,eax                  ; copy over the 16.16 value
  Add  eax,[_HorizontalDC]       ; increment
                                ;
  And  ebx,0003f0000h           ;
  Inc  edi                      ;
                                ;
  Shr  ebx,8                    ; mask for table lookup
                                ;
  Mov  dl,[esi+ebx]             ; get shade table color
                                ;
  Dec  ecx                      ;
  Mov  [edi-1],dl               ; store into screen
  Jnz @@FillLoop                ;
                                ;
 @@SkipFill:                    ;
                                ;
  Mov  edx,[_YAddress]           ; go to the next scanline
  Add  edx,_LineSize           ;
  Mov  [_YAddress],edx           ; YAddress += LineSize
                                ;
  Mov  eax,[Side&C]             ; increment the color span
  Add  eax,[Side&DC]            ;
  Mov  [Side&C],eax             ;
                                ;
  Mov  eax,[Left&X]             ;
  Mov  ebx,[Right&X]            ;
  Add  eax,[Left&DX]            ;
  Add  ebx,[Right&DX]           ;
  Mov  [Left&X],eax             ; LeftX += LeftDX
  Mov  [Right&X],ebx            ; RightX += RightDX
                                ;
  Jmp @@HeightLooper            ;
                                ;
 @@ExitHeight:                  ;
EndM                            ;
                                ;
;===========================================================================
; CullPolys
;===========================================================================
_GouraudPolygonASM Proc Uses edi eax,_Triangle:DWord, _VirtualAddress:DWord, _LineSize:DWord
                                ;
  Pushad

  Mov  edi,_Triangle           ;
                                ;
  Mov   ebx,[edi].pColor        ;
  And   ebx,0ffh                ;
  Mov   [_Color],ebx             ;
                                ;
include affasm.inc              ; include affine interpolation

  Popad
                                ;
  Ret                           ;
                                ;
_GouraudPolygonASM EndP          ;


CODE ENDS
END


